<?php
// TOOLSET TO CREATE AND MANAGE SUBSCRIPTIONS THROUGH (WOOCOMMERCE ..) ORDERS

class pcpp_subscriptions {
	public $errors = false; // static resource for errors reporting
	
	// static resources to store trial period duration
	public $trial_duration = false; 
	public $trial_dur_type = 'days'; 
	
	
	/* KNOW IF THERE'S TRIAL PERIOD - used also to setup static properties 
	 * @return (bool)
	 */
	public function is_trial_enabled() {
		if($this->trial_duration === false) {
			$this->trial_duration = (int)get_option('pcpp_trial_duration');
			$this->trial_dur_type = get_option('pcpp_trial_type', 'days');
		}
		
		return ($this->trial_duration) ? true : false;
	}
	
	
	
	/*
	 * get available plans
	 * @param (bool) $strip_no_form = whether to exclude plans not to be shown in regisrtation forms
	 * @param (bool) $strip_no_renew = whether to exclude plans not to be shown in renewal form
	 * @plan_id (int) eventually returns only specific plan name
	 *
	 * @return (array/string) plans associative array (plan_id => name) or the plan name requested
	 */
	public function get_plans($strip_no_form = true, $strip_no_renew = false, $plan_id = false) {
		include_once(PCPP_DIR .'/functions.php');
		$GLOBALS['pcpp_is_querying plans'] = true;
		
		// check for cache - only for stripped fields
		if(!$strip_no_form && !$strip_no_renew && !$plan_id && isset($GLOBALS['pcpp_avail_plans'])) {
			unset($GLOBALS['pcpp_is_querying plans']);
			return $GLOBALS['pcpp_avail_plans'];
		}
		
		$plans = array();
		if(pcpp_avail_ecomm() === array()) { // no e-comm installed
			unset($GLOBALS['pcpp_is_querying plans']);
			return $plans;
		} 
		
		
		// get subject
		$chosen = get_option('pcpp_ecomm_to_use', pcpp_avail_ecomm(true));
	
		if($chosen == 'woocomm') {
			$args = array(
				'post_type' 		=> 'product',
				'post_status' 		=> 'publish',
				'posts_per_page' 	=> 100,
				'orderby'			=> 'menu_order',
				'order'				=> 'ASC',
				'tax_query' 		=> array(
					array(
						'taxonomy' => 'product_type',
						'field'    => 'slug',
						'terms'    => 'pvtcontent_plan', 
					),
				),
			);
			
			$query = new WP_Query($args);
			foreach($query->posts as $post) {
				
				if($strip_no_form && get_post_meta($post->ID, 'pcpp_not_in_forms', true)) {
					continue;
				}
				
				if($strip_no_renew && get_post_meta($post->ID, 'pcpp_not_in_renew', true)) {
					continue;
				}
				
				$plans[ $post->ID ] = $post->post_title;	
			}
		}
		
		
		/* PCPP FILTER - allows fetched plans array alteration - passes an associative array('plan-id' =>plan-name) and $strip_no_form, $strip_no_renew */
		$plans = apply_filters('pcpp_fetched_plans', $plans, $strip_no_form, $strip_no_renew);
		
		
		// set cache - only for stripped fields
		if(!$strip_no_form && !$plan_id){
			$GLOBALS['pcpp_avail_plans'] = $plans;	
		}
		
		unset($GLOBALS['pcpp_is_querying plans']);
		return ($plan_id && isset($plans[$plan_id])) ? $plans[$plan_id] : $plans; 
	}
	
	
	
	/*
	 * GET PLAN'S DATA
	 * @param (int) $plan_id = plan id (custom taxonomy's post id) 
	 * @return (array) associative array containing plan's data
	 */ 
	public function get_plan_data($plan_id) {
		include_once(PCPP_DIR .'/functions.php');
		$data = array();
		
		// get price
		if(PCPP_ECOMMERCE == 'woocomm') {
			$product = wc_get_product($plan_id);	
			$data['price'] = $product->get_price();
			
			$data['currency_price'] = strip_tags($product->get_price_html());
		}
		else {
			$data['price'] = 0;	
			$data['currency_price'] = '\$0';
		}
		
		// get duration
		$duration = (int)get_post_meta($plan_id, 'pcpp_plan_duration', true);
		$dur_type = get_post_meta($plan_id, 'pcpp_plan_dur_type', true);
		
		$data['duration'] 		= $duration;
		$data['duration_type'] 	= $dur_type;
		$data['duration_secs'] 	= $duration * pcpp_periods_to_sec($dur_type);
		$data['duration_txt'] 	= (!$data['duration_secs']) ? __('lifetime') : $duration.' '.pcpp_sing_plur_switch($dur_type, $duration);
		
		// get description 
		$data['txt'] = get_post_field('post_content', pcpp_wpml_get_trans_id($plan_id));
		
		return $data;
	}
	
	
	
	/* CREATE ORDER 
	 * @param (int) $user_id - privatecontent user ID to associate
	 * @param (int) $plan_id - plan to associate to the order (if empty will be queried)
	 * @param (array) $user_data - associative array containing data useful for the order creation (if empty - a query will be performed)
	 * @param (string) $coupon - discount coupon to eventually use during order creation
	 *
	 * @return (int/bool) order ID if successfully created - otherwise false
	 */
	public function create_order($user_id, $plan_id = false, $user_data = array(), $coupon = false) {
		global $pc_users, $pc_wp_user;
		
		// where to query also associated plan ID
		if(empty($plan_id)) {
			global $pc_meta;
			$plan_id = (isset($user_data['pcpp_plan'])) ? $user_data['pcpp_plan'] : $pc_meta->get_meta($user_id, 'pcpp_plan'); 	
		}
		
		// whether to get user's data
		if(empty($user_data)) {
			$user_data = $pc_users->get_user($user_id, $args = array('to_get' => array('name', 'surname', 'username', 'email', 'tel', 'wp_user_id', 'psw')));	
		}
		else {
			if(!isset($user_data['psw'])) {
				$user_data['psw'] = $pc_users->get_user_field($user_id, 'psw');	
			}
		}

		// woocommerce order
		if(PCPP_ECOMMERCE == 'woocomm') {
			$args = array(
				'customer_id' => (empty($user_data['wp_user_id'])) ? false : $user_data['wp_user_id'], // if WP-synced - attach user
			);
			
			// if free - is already paid
			$product = wc_get_product($plan_id);	
			$price = $product->get_price();
			
			if(!$price) {
				$args['status'] = 'completed';
			}
			
			$order = wc_create_order($args);
			$order_id = trim(str_replace('#', '', $order->get_order_number())); 
			
			// check for errors
			if(is_wp_error($order)) {
				$this->errors = $order->get_error_message();
				return false;
			}
			
			
			// since July 2018 WooComm changes user psw - then have to force it
			if(!empty($user_data['wp_user_id'])) {
				global $wpdb;
				
				$pc_wp_user->sync_psw_to_wp($user_data['psw'], (int)$user_data['wp_user_id']);	
				$wpdb->update(PC_USERS_TABLE, array('psw' => $user_data['psw']),  array('id' => (int)$user_id));
			}
			

			// set billing address
			$billing = array(
				'first_name' => $user_data['name'],
				'last_name'  => $user_data['surname'],
				'company'    => $this->billing_cust_field($user_id, 'pcpp_bill_company_f'),
				'email'      => $user_data['email'],
				'phone'      => $user_data['tel'],
				'address_1'  => $this->billing_cust_field($user_id, 'pcpp_bill_address_f'),
				'city'       => $this->billing_cust_field($user_id, 'pcpp_bill_city_f'),
				'postcode'   => $this->billing_cust_field($user_id, 'pcpp_bill_postcode_f'),
				'country'    => $this->billing_cust_field($user_id, 'pcpp_bill_country_f'),
				'state'      => $this->billing_cust_field($user_id, 'pcpp_bill_state_f'),
			);
			$order->set_address($billing, 'billing');
			
			
			// update user's billing and shipping address 
			if(!empty($user_data['wp_user_id'])) {
				$wc_user_meta = array(
					'billing_company'	=> $this->billing_cust_field($user_id, 'pcpp_bill_company_f'),
					'billing_address_1'	=> $this->billing_cust_field($user_id, 'pcpp_bill_address_f'),
					'billing_city'     	=> $this->billing_cust_field($user_id, 'pcpp_bill_city_f'),
					'billing_postcode'	=> $this->billing_cust_field($user_id, 'pcpp_bill_postcode_f'),
					'billing_country' 	=> $this->billing_cust_field($user_id, 'pcpp_bill_country_f'),
					'billing_state'		=> $this->billing_cust_field($user_id, 'pcpp_bill_state_f'),
					'billing_phone' 	=> $user_data['tel'],
					
					'shipping_company'		=> $this->billing_cust_field($user_id, 'pcpp_bill_company_f'),
					'shipping_address_1'	=> $this->billing_cust_field($user_id, 'pcpp_bill_address_f'),
					'shipping_city'     	=> $this->billing_cust_field($user_id, 'pcpp_bill_city_f'),
					'shipping_postcode'		=> $this->billing_cust_field($user_id, 'pcpp_bill_postcode_f'),
					'shipping_country' 		=> $this->billing_cust_field($user_id, 'pcpp_bill_country_f'),
					'shipping_state'		=> $this->billing_cust_field($user_id, 'pcpp_bill_state_f'),
					'shipping_phone' 		=> $user_data['tel'],
				);
				
				foreach($wc_user_meta as $meta_key => $meta_value) {
					update_user_meta($user_data['wp_user_id'], $meta_key, $meta_value );
				}
			}
			
			
			// if there's a coupon
			if($price && !empty($coupon)) {
				$coupon_inst = new WC_Coupon($coupon);
				$coup_amount = $coupon_inst->get_amount();
				
				if(!empty($coup_amount)) {
					if($coupon_inst->is_valid_for_product($product) && $price >= $coupon_inst->get_minimum_amount() && (!$coupon_inst->get_maximum_amount() || $price <= $coupon_inst->get_maximum_amount()) ) {	
					
						// set a discout on this order equal to coupon value
						$discount_amount = $coupon_inst->get_discount_amount($price, null, true);
						if($discount_amount) {
							
							$args = array(
								"totals" => array(
									'subtotal' => $price,
									'total' => $price - $discount_amount
								)
							);     
							$order->add_product( wc_get_product( $plan_id ), 1, $args); // 1 == quantity	
							$coupon_used = true;
							
							$coupon_inst->increase_usage_count($user_data['email']); // increase coupon usage count
							$order->add_order_note('<p>'. __('Order discounted using "'. $coupon .'" coupon', 'pcpp_ml') .'</p>');
							
							// 100% coupon? - set order as paid
							if(!($price - $discount_amount)) {
								$order->update_status('completed', '', true);	
								$GLOBALS['pcpp_free_order_due_to_coupon'] = true;
							}
						}
					}
				}
			}
			
			
			// no coupon - add product normally
			if(!isset($coupon_used)) {
				$order->add_product( wc_get_product( $plan_id ), 1); // 1 == quantity		
			}
			
			// calculate total
			$order->calculate_totals();

			// add meta to mark it as pvtontent order
			add_post_meta($order_id, 'pvtcontent_order', 1);
			add_post_meta($order_id, 'pvtcontent_user_id', $user_id);
			
			//var_dump($order);
			return $order_id;
		}
		
		return false;	
	}
	
	
	// return user meta to fill specific billing data
	public function billing_cust_field($user_id, $field) {
		// woocomm user orders list - abort
		if(isset($_GET['post_type']) && $_GET['post_type'] = 'shop_order' && isset($_GET['_customer_user'])) {return false;}
		
		if(isset($GLOBALS['pcpp_billing_custom_fields']) && isset($GLOBALS['pcpp_billing_custom_fields'][$user_id])) {
			return (isset($GLOBALS['pcpp_billing_custom_fields'][$user_id][$field])) ? $GLOBALS['pcpp_billing_custom_fields'][$user_id][$field] : '';
		}
		
		global $pc_users;
		$fields = array(
			'pcpp_bill_company_f' 	=> get_option('pcpp_bill_company_f'),
			'pcpp_bill_address_f' 	=> get_option('pcpp_bill_address_f'),	
			'pcpp_bill_city_f' 		=> get_option('pcpp_bill_city_f'),	
			'pcpp_bill_state_f'		=> get_option('pcpp_bill_state_f'),
			'pcpp_bill_country_f'	=> get_option('pcpp_bill_country_f'),
			'pcpp_bill_postcode_f'	=> get_option('pcpp_bill_postcode_f'),		
		);
		foreach($fields as $index => $val) {
			if(empty($val)) {unset($fields[$index]);}	
		}
		
		// no fields associated 
		if(empty($fields)) {
			$GLOBALS['pcpp_billing_custom_fields'] = array();
			return '';	
		}
		
		$u_data = $pc_users->get_user($user_id, array('to_get' => array_values($fields) ));	
		$GLOBALS['pcpp_billing_custom_fields'] = array();
		$GLOBALS['pcpp_billing_custom_fields'][$user_id] = array();

		foreach($fields as $pcpp_key => $db_key) {
			$GLOBALS['pcpp_billing_custom_fields'][$user_id][$pcpp_key] = (empty($db_key)) ? '' : $u_data[$db_key];	
		}
		
		return (isset($GLOBALS['pcpp_billing_custom_fields'][$user_id][$field])) ? $GLOBALS['pcpp_billing_custom_fields'][$user_id][$field] : ''; 
	}



	// given the order ID - returns checkout URL
	public function checkout_url($order_id) {
		if(PCPP_ECOMMERCE == 'woocomm') {
			$url = 	wc_get_checkout_url() .'/order-pay/'. $order_id .'?pay_for_order=true&key='. get_post_meta($order_id, '_order_key', true);
		}
		
		return $url;
	}
	
	
	
	/* SETUP TRIAL PERIOD META - given user ID set trial metas if there's any
	 * @param (int) $user_id 
	 * @return (bool)
	 */
	public function setup_trial_meta($user_id) {
		include_once(PCPP_DIR .'/functions.php');
		global $pc_meta;
		
		if($this->is_trial_enabled()) {
			$pc_meta->update_meta($user_id, 'pcpp_subscr_status', 'trial');	
			$pc_meta->update_meta($user_id, 'pcpp_trial_ending', pcpp_get_ending_date($this->trial_duration, $this->trial_dur_type) );		
		}	
		return true;
	}
	


	/* GET ORDER object */
	public function get_order($order_id) {
		if(PCPP_ECOMMERCE == 'woocomm') {
			$order = new WC_Order($order_id);	
		}
		
		return $order;
	}
	
	
	
	/* GET ORDER STATUS - given order ID */
	public function get_order_status($order_id) {
		$status = ''; // no status by default
		
		if(PCPP_ECOMMERCE == 'woocomm') {
			$order = $this->get_order($order_id);
			$wc_status = get_post_status($order_id);
			
			switch($wc_status) {
				case 'wc-pending' : 
				case 'wc-on-hold' :
				case 'wc-processing' : 
					$status = 'unpaid'; break;	
					
				case 'wc-cancelled' : 
				case 'wc-refunded' : 
				case 'wc-failed' : 
					$status = 'expired'; break;	
					
				case 'wc-completed' : 
					$status = 'active'; break;			
			}
		}
				
		//trial/unpaid/expired/active
		return $status;
	}
	
	
	
	/* 
	 * CANCEL ORDER - given order ID cancel it if isn't paid or already cancelled and eventually add a note
	 */
	public function cancel_order($order_id, $note = false) {
		if(!in_array($this->get_order_status($order_id), array('active', 'expired'))) {
			if(PCPP_ECOMMERCE == 'woocomm') {
				$order = $this->get_order($order_id);
				$order->update_status('cancelled', $note.' -', true);	
			}
		}
		
		return true;	
	}
	
	
	
	/* MANAGE USER ORDERS - add or remove orders related to a user
	 * @param (int) $user_id = privateContent user id
	 * @param (int) $order_id = order ID to add or remove
	 * @param (string) $action = could be "add" or "remove"
	 * @param (bool) $set_new_main = whether to set new main user order
	 *
	 * @return (bool) true if operation was successful - otherwise false
	 */ 
	public function manag_user_orders($user_id, $order_id, $action, $set_new_main = true) {
		if(!in_array($action, array('add', 'remove'))) {return false;}
		global $pc_meta;
		
		// get meta
		$db_val = $pc_meta->get_meta($user_id, 'pcpp_orders');
		if(empty($db_val) || !is_array($db_val)) {
			$db_val	= array();
		}
		
		// if add
		if($action == 'add') {
			$db_val[] = (int)$order_id;
			asort($db_val);
			$new_val = $db_val;
			
			if($set_new_main) {
				$this->set_user_main_order($user_id, $order_id);
			}
		}
		
		// remove
		else {
			$new_val = array();
			foreach($db_val as $db_order_id) {
				if($db_order_id != $order_id) {$new_val[] = $db_order_id;}	
			}
			
			if(count($new_val)) {
				asort($new_val);
				
				if($set_new_main) {
					$this->set_user_main_order($user_id, end($new_val));
				}
			}
			else {
				// no orders left - set no_act_order status
				$pc_meta->update_meta($user_id, 'pcpp_subscr_status', 'no_act_order');	
				$pc_meta->update_meta($user_id, 'pcpp_plan','');
				$pc_meta->update_meta($user_id, 'pcpp_subscr_end', '');
			}
		}
		
		$result = (bool)$pc_meta->update_meta($user_id, 'pcpp_orders', $new_val);
		return $result; 
	}


	 
	/* SET USER MAIN ORDER - will be taken into consideration to manage subscription expiration and subscription status
	 * @param (int) $user_id
	 * @param (int) $order_id - new main order ID
	 */
	public function set_user_main_order($user_id, $order_id) {
		global $pc_meta;
		
		$status = $this->get_order_status($order_id); // get order status
		$plan_id = $this->get_order_plan_id($order_id); // associated plan
		$expiration = $this->order_plan_expiration($order_id); // ending date
		 	
		// check if already expired	
		if($expiration != 'unlimited' && $expiration < current_time('timestamp'))	{
			$status = 'expired';	
		}
			
		// set metas		
		$pc_meta->update_meta($user_id, 'pcpp_plan', $plan_id);
		$pc_meta->update_meta($user_id, 'pcpp_active_order', $order_id);
		$pc_meta->update_meta($user_id, 'pcpp_subscr_end', $expiration);
		$pc_meta->update_meta($user_id, 'pcpp_subscr_status', $status);		
				
		return true;
	}
	
	
	/* CHECK USER'S SUBSCRIPTION EXPIRATION DATE 
	 * if expired, update status 
	 * checks also trial - if expired set to unpaid and setup pcpp_expired_trial global
	 *
	 * @param (int) $user_id
	 * @return (bool/string) 
	 	- true if not associated to PCPP
		- status string
	 */
	public function check_user_subscr_expir($user_id) {
		global $pc_users, $pc_meta;
		
		// get user meta
		$to_get = array('pcpp_active_order', 'pcpp_subscr_status', 'pcpp_trial_ending', 'pcpp_subscr_end');
		$ud = $pc_users->get_user($user_id, array('to_get' => $to_get));
		
		// if no subscription status or end specified - grant access
		if(empty($ud['pcpp_subscr_status']) && empty($ud['pcpp_subscr_end'])) {return true;}
		
		
		// expired or pending payment
		if(in_array($ud['pcpp_subscr_status'], array('unpaid', 'no_act_order', 'expired'))) {return $ud['pcpp_subscr_status'];}
		
		// trial period - check expiration and eventually set new status
		elseif($ud['pcpp_subscr_status'] == 'trial') {
			if((int)$ud['pcpp_trial_ending'] <= (int)current_time('timestamp')) {
				
				// get user active order and set its status or no order found - set to expired to allow new order creation
				$new_status = (empty($ud['pcpp_active_order'])) ? 'expired' : $this->get_order_status($ud['pcpp_active_order']);
				$pc_meta->update_meta($user_id, 'pcpp_subscr_status', $new_status);

				$GLOBALS['pcpp_expired_trial'] = true; 
				return $new_status;		
			}
			else {return 'trial';}	
		}
		
		// active - check expiration
		elseif($ud['pcpp_subscr_status'] == 'active') {
			if($ud['pcpp_subscr_end'] != 'unlimited' && (int)$ud['pcpp_subscr_end'] <= (int)current_time('timestamp')) {
				$pc_meta->update_meta($user_id, 'pcpp_subscr_status', 'expired'); // update status to expired
				return 'expired';
			}	
			else {return 'active';}
		}
		
		return false;
	}
	
	
	/* GET ORDER'S PLAN - given order id, returns related pcpp plan id */
	public function get_order_plan_id($order_id) {
		if(PCPP_ECOMMERCE == 'woocomm') {
			$order = new WC_Order($order_id);
			$items = array_values($order->get_items());
			
			$plan_id = (version_compare(PHP_VERSION, '3.0.0', '>=')) ? $items[0]['product_id'] : $items[0]['item_meta']['_product_id'][0];
		}
		
		return $plan_id;
	}
	
	
	/* GET PLAN EXPIRATION TIME - RELATED TO ORDER DATE
	 * @param (int) $order_id - order ID
	 * @param (string) $starting_date = when to start period (order = order submission date - now = current date)
	 * @return (int) expiration date in time format
	 */
	public function order_plan_expiration($order_id, $starting_date = 'order') {
		include_once(PCPP_DIR .'/functions.php');
		
		if(PCPP_ECOMMERCE == 'woocomm') {
			$order = new WC_Order($order_id);

			// get order's plan duration
			$plan_data = $this->get_plan_data( $this->get_order_plan_id($order_id));
			
			// if unlimited
			if(empty($plan_data['duration'])) {
				$exp = 'unlimited';	
			} else {	
				$start_from = ($starting_date == 'order') ? get_post_time('Y-m-d H:i:s', $gmt = false, $order_id) : false;	
				$exp = pcpp_get_ending_date($plan_data['duration'], $plan_data['duration_type'], $to_get = 'time', $start_from); 
			}
		}
		
		return $exp; 	
	}
	
	

	/* KNOW IF AND ORDER BELONGS TO PVTCONTENT */
	public function is_pvtcontent_order($order_id) {
		return (get_post_meta($order_id, 'pvtcontent_order', true)) ? true : false;		
	}
	
}
$GLOBALS['pcpp_subscr'] = new pcpp_subscriptions();
